/*
  ==============================================================================

    ResizableImage.h
    Created: 31 Mar 2016 11:36:17am
    Author:  George Scrivener

  ==============================================================================
*/

#ifndef RESIZABLEIMAGE_H_INCLUDED
#define RESIZABLEIMAGE_H_INCLUDED

#include "JuceHeader.h"

using namespace juce;

class ResizableImage
{
public:
    struct ImageInsets
    {
        ImageInsets();
        ImageInsets(int leftInset, int rightInset, int topInset, int bottomInset, int overscalingFactor = 4);
        
        bool operator==(const ImageInsets& other) const;
        bool operator!=(const ImageInsets& other) const;
        
        int leftInset ;
        int rightInset;
        int topInset;
        int bottomInset;
        
        int overscalingFactor;
        
        bool isZero()  const;
    };
    
    enum ResizeMode {
        DontResize,
        OnlyDrawOutside,
        StretchInsde
    };
    
    ResizeMode resizeMode = StretchInsde;
    
    ResizableImage();
    ~ResizableImage();
    ResizableImage(const juce::Image& image, ImageInsets insets);
    ResizableImage(const char * data, int
                   dataSize, int leftInset, int rightInset, int topInset, int bottomInset,
                   int overscalingFactor = 4);
    
    bool operator==(const ResizableImage& other) const;
    bool operator!=(const ResizableImage& other) const;
    
    bool isValid() const;
    
    void setImage(const juce::Image& Image, ImageInsets insets);
    void setImage(const char * data, int dataSize, ResizableImage::ImageInsets insets);
    void setImage(const char * data, int dataSize, int leftInset, int rightInset, int topInset, int bottominset);
    
    void drawInRect(const Graphics& g, Rectangle<int> rect) const;
    void drawInRect(const Graphics& g, int x, int y, int width, int height) const;
    
    int imageHeight() const;
    int imageWidth() const;
    
    const ImageInsets& insets() const;
    ResizableImage& operator= (const ResizableImage& other);
    
    Image _image;
    ImageInsets _imageInsets;
    
    int limitX(const int iNumberToLimit) const;
    int limitY(const int iNumberToLimit) const;
    int limitWidth(const int iNumberToLimit) const;
    int limitHeight(const int iNumberToLimit) const;
private:
    
    
    Image _imageTop;
    Image _imageBottom;
    Image _imageRight;
    Image _imageLeft;
    Image _imageTopLeft;
    Image _imageTopRight;
    Image _imageBottomLeft;
    Image _imageBottomRight;
    void createAllImages();
    
};



class ResizableImageComponent : public Component {
public:
    
    void setImage(const char * data, int dataSize, ResizableImage::ImageInsets insets)
    {
        image.setImage(data, dataSize, insets);
        repaint();
    }
   
    void setImage(const char * data, int dataSize, int leftInset, int rightInset, int topInset, int bottominset)
    {
        image.setImage(data, dataSize, leftInset, rightInset, topInset, bottominset);
        repaint();
    }
   
    void setImage(const ResizableImage& image_)
    {
        image = image_;
        repaint();
    }
    
    void paint (juce::Graphics &g) {
        if (image.isValid()) {
            image.drawInRect(g, getLocalBounds());
        }
    }
    
    ResizableImage image;
};


class ResizableImageLabel : public Label
{
public:
    ResizableImageLabel()
    :   Label("Testing_Bapuji", "Testing")
    {
        setJustificationType(Justification::centred);
        setFont(Font(25));
        setColour(Label::backgroundColourId, Colours::transparentBlack);
    }
    
    void paint(Graphics& g) override
    {
        
        auto rect = getLocalBounds();
//        auto x = rect.getX();
//        auto y = rect.getY();
//        auto width = rect.getWidth();
//        auto height = rect.getHeight();
//        if (!(width > 0 && height > 0) || (!backgroundImage._image.isValid())) { return; }
//
//
//
//        int leftInset = backgroundImage._imageInsets.leftInset / backgroundImage._imageInsets.overscalingFactor;
//        int rightInset = backgroundImage._imageInsets.rightInset / backgroundImage._imageInsets.overscalingFactor;
//        int topInset = backgroundImage._imageInsets.topInset / backgroundImage._imageInsets.overscalingFactor;
//        int bottomInset = backgroundImage._imageInsets.bottomInset / backgroundImage._imageInsets.overscalingFactor;
//
//        int imageWidth = backgroundImage._image.getWidth();
//        int imageHeight = backgroundImage._image.getHeight();
//        int centreWidth = width - leftInset - rightInset;
//        int centreHeight = height - topInset - bottomInset;
//
//        int imageCentreWidth = imageWidth - backgroundImage._imageInsets.leftInset - backgroundImage._imageInsets.rightInset;
//        int imageCentreHeight = imageHeight - backgroundImage._imageInsets.topInset - backgroundImage._imageInsets.bottomInset;
//
//        // ============================== Draw Centre ==============================
//        if (centreWidth > 0 && centreHeight > 0 && imageCentreWidth > 0 && imageCentreHeight > 0) {
//            switch (backgroundImage.resizeMode) {
//                case ResizableImage::OnlyDrawOutside:
//                    break;
//
//                case ResizableImage::StretchInsde:
//                                g.drawImage(backgroundImage._image,
//                                            0,
//                                            0,
//                                            rect.getWidth(),
//                                            rect.getHeight(),
//                                            0,
//                                            0,
//                                            backgroundImage._image.getWidth(),
//                                            backgroundImage._image.getHeight());
//
//                    //g.drawImageAt(_image,  x + leftInset,y + topInset);
////                    g.drawImage(backgroundImage._image,
////                                x + leftInset,
////                                y + topInset,
////                                centreWidth,
////                                centreHeight,
////                                backgroundImage._imageInsets.leftInset,
////                                backgroundImage._imageInsets.topInset,
////                                imageCentreWidth,
////                                imageCentreHeight);
//                    break;
//
//                case ResizableImage::DontResize:
//                    /// This case should have already been handled and should have returned
//                    jassertfalse;
//                    break;
//            }
//        }
        
        // ============================== Draw Edges ==============================
        
    //    if (centreWidth > 0) {
    //        // Top
    //        g.drawImage(_imageTop,
    //                    x + rightInset,
    //                    y,
    //                    centreWidth,
    //                    topInset,
    //                    _imageInsets.leftInset,
    //                    0,
    //                    imageCentreWidth,
    //                    _imageInsets.topInset);
    //
    //        // Bottom
    //        g.drawImage(_imageBottom,
    //                    x + leftInset,
    //                    y + height - bottomInset,
    //                    centreWidth,
    //                    bottomInset,
    //                    _imageInsets.leftInset,
    //                    imageHeight - _imageInsets.bottomInset,
    //                    imageCentreWidth,
    //                    _imageInsets.bottomInset);
    //    }

    //    if (centreHeight > 0) {
    //        // Right
    //        g.drawImage(_imageRight,
    //                    x + width - rightInset,
    //                    y + topInset,
    //                    rightInset,
    //                    centreHeight,
    //                    imageWidth - _imageInsets.rightInset,
    //                    _imageInsets.topInset,
    //                    _imageInsets.rightInset,
    //                    imageCentreHeight);
    //
    //        // Left
    //        g.drawImage(_imageLeft,
    //                    x,
    //                    y + topInset,
    //                    leftInset,
    //                    centreHeight,
    //                    0,
    //                    _imageInsets.topInset,
    //                    _imageInsets.leftInset,
    //                    imageCentreHeight);
    //    }
    //
    //    // ============================== Draw Corners ==============================
    //    // Top Left
    //    g.drawImage(_imageTopLeft,
    //                x,
    //                y,
    //                leftInset,
    //                topInset,
    //                0,
    //                0,
    //                _imageInsets.leftInset,
    //                _imageInsets.topInset);
    //
    //    // Top Right
    //    g.drawImage(_imageTopRight,
    //                x + width - rightInset,
    //                y,
    //                rightInset,
    //                topInset,
    //                imageWidth - _imageInsets.rightInset,
    //                0,
    //                _imageInsets.rightInset,
    //                _imageInsets.topInset);
    //
    //    // Bottom Left
    //    g.drawImage(_imageBottomLeft,
    //                x,
    //                y + height - bottomInset,
    //                leftInset,
    //                bottomInset,
    //                0,
    //                imageHeight - _imageInsets.bottomInset,
    //                _imageInsets.leftInset,
    //                _imageInsets.bottomInset);
    //
    //    // Bottom Right
    //    g.drawImage(_imageBottomRight,
    //                x + width - rightInset,
    //                y + height - bottomInset,
    //                rightInset,
    //                bottomInset,
    //                imageWidth - _imageInsets.rightInset,
    //                imageHeight - _imageInsets.bottomInset,
    //                _imageInsets.rightInset,
    //                _imageInsets.bottomInset);
        
        backgroundImage.drawInRect(g, getLocalBounds());
        
//            g.drawImage(backgroundImage._image,
//                        0,
//                        0,
//                        rect.getWidth(),
//                        rect.getHeight(),
//                        0,
//                        0,
//                        backgroundImage._image.getWidth(),
//                        backgroundImage._image.getHeight());
        
        g.setColour(Colours::red);
        g.drawText(getText(), rect, Justification::centred);
    }
    
    ResizableImage backgroundImage;
};

#endif  // RESIZABLEIMAGE_H_INCLUDED
