
#ifndef ResizableImageLabel_H
#define ResizableImageLabel_H

#include "JuceHeader.h"

using namespace juce;

class ResizableImage
{
public:
    //=================================================
    struct ImageInsets
    {
        ImageInsets();
        ImageInsets(int leftInset, int rightInset, int topInset, int bottomInset, int overscalingFactor = 4);
        int leftInset ;
        int rightInset;
        int topInset;
        int bottomInset;
        
        int overscalingFactor;
        
        bool isZero()  const;
    };
    
    enum ResizeMode
    {
        DontResize,
        OnlyDrawOutside,
        StretchInsde
    };
    
    ResizeMode resizeMode = StretchInsde;
    //=================================================
    
    ResizableImage();
    ~ResizableImage();

    void setImage(const juce::Image& Image, ImageInsets insets);
   
    void drawInRect(const Graphics& g, Rectangle<int> rect) const;
    void drawInRect(const Graphics& g, int x, int y, int width, int height) const;

private:
    
    Image _image;
    ImageInsets _imageInsets;
};


class ResizableImageLabel : public Label
{
public:
    ResizableImageLabel(const String& name)
    :   Label(name, name)
    {
        setJustificationType(Justification::centred);
        setFont(Font(25));
        setColour(Label::backgroundColourId, Colours::transparentBlack);
    }
    
    void paint(Graphics& g) override
    {
        auto rect = getLocalBounds();
        _backgroundImage.drawInRect(g, rect);
        g.setColour(Colours::white);
        g.drawText(getText(), rect, Justification::centred);
    }
    
    ResizableImage _backgroundImage;
};

#endif  // ResizableImageLabel_H
